from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.services.views import ServiceAdd

import json
import mock


class ServiceAddViewTest(TestCase):
    @mock.patch("requests.post")
    @mock.patch("requests.get")
    def test_post(self, get, post):
        get.return_value = mock.Mock(status_code=200)
        data = {"name": "name", "team": "team"}
        request = RequestFactory().post("/", data)
        request.session = {"tsuru_token": "admin"}
        response = ServiceAdd.as_view()(request, service_name="service")
        self.assertEqual(302, response.status_code)
        self.assertEqual(reverse('service-list'), response.items()[1][1])
        post.assert_called_with(
            '{0}/services/instances'.format(settings.TSURU_HOST),
            headers={'authorization': 'admin'},
            data=json.dumps({"name": "name", "team": "team", "service_name": "service"}))

    @mock.patch("requests.get")
    def test_get(self, get):
        get.return_value = mock.Mock(status_code=200)
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        response = ServiceAdd.as_view()(request, service_name="service")
        self.assertEqual(200, response.status_code)



from django.test import TestCase
from django.test.client import RequestFactory
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from tsuru_dashboard import settings

from tsuru_dashboard.apps.views import Unlock

import mock


class UnlockTestCase(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/name/remove")
        self.request.session = {"tsuru_token": "admin"}

    @mock.patch('requests.delete')
    @mock.patch('requests.get')
    @mock.patch("django.contrib.messages.error")
    def test_should_returns_404_when_app_does_not_exists(self, msg_mock, get, delete):
        get.return_value = mock.Mock(status_code=200)
        delete.return_value = mock.Mock(status_code=404, text="app not found")

        response = Unlock.as_view()(self.request, name="appname")

        self.assertEqual(302, response.status_code)
        msg_mock.assert_called_with(self.request, 'app not found', fail_silently=True)

    @mock.patch('requests.delete')
    @mock.patch('requests.get')
    @mock.patch("django.contrib.messages.success")
    def test_should_redirect_to_the_settings(self, msg_mock, get, delete):
        get.return_value = mock.Mock(status_code=200)
        delete.return_value = mock.Mock(status_code=200)

        response = Unlock.as_view()(self.request, name="appname")

        self.assertEqual(302, response.status_code)
        self.assertIsInstance(response, HttpResponseRedirect)
        self.assertEqual(reverse('app-settings', args=["appname"]), response['Location'])

        url = "{}/apps/appname/lock".format(settings.TSURU_HOST)
        delete.assert_called_with(url, headers={'authorization': 'admin'})
        msg_mock.assert_called_with(self.request, u'App was successfully unlocked', fail_silently=True)



from tsuru_dashboard import settings

import requests
import json
import datetime
import re


class MetricNotEnabled(Exception):
    pass


def get_backend(app, token, date_range=None, process_name=None):
    headers = {'authorization': token}
    url = "{}/apps/{}/metric/envs".format(settings.TSURU_HOST, app["name"])
    response = requests.get(url, headers=headers)

    if response.status_code == 200:
        data = response.json()
        if "METRICS_ELASTICSEARCH_HOST" in data:
            return ElasticSearch(
                url=data["METRICS_ELASTICSEARCH_HOST"],
                app=app["name"],
                process_name=process_name,
                date_range=date_range
            )

    if "envs" in app and "ELASTICSEARCH_HOST" in app["envs"]:
        return ElasticSearch(
            url=app["envs"]["ELASTICSEARCH_HOST"],
            app=app["name"],
            process_name=process_name,
            date_range=date_range
        )

    raise MetricNotEnabled


NET_AGGREGATION = {
    "units": {
        "terms": {
            "field": "host"
        },
        "aggs": {
            "delta": {
                "scripted_metric": {
                    "init_script": """
_agg['max'] = [ts: 0, val: null]
_agg['min'] = [ts: 0, val: null]
""",
                    "map_script": """
ts = doc['@timestamp'][0]
val = doc['value'][0]
if (ts > _agg.max.ts) {
    _agg.max.ts = ts
    _agg.max.val = val
}
if (_agg.min.ts == 0 || ts < _agg.min.ts) {
    _agg.min.ts = ts
    _agg.min.val = val
}
""",
                    "reduce_script": """
max = null
min = null
for (a in _aggs) {
    if (max == null || a.max.ts > max.ts) {
        max = a.max
    }
    if (min == null || a.min.ts < min.ts) {
        min = a.min
    }
}
dt = max.ts - min.ts
if (dt > 0) {
    return ((max.val - min.val)/1024)/(dt/1000)
} else {
    return 0
}
"""
                }
            }
        }
    }
}


class ElasticSearch(object):
    def __init__(self, url, app, process_name=None, date_range="1h"):
        if date_range == "1h":
            self.index = ".measure-tsuru-{}".format(datetime.datetime.utcnow().strftime("%Y.%m.%d"))
        else:
            self.index = ".measure-tsuru-{}.*".format(datetime.datetime.utcnow().strftime("%Y"))
        self.app = app
        self.url = url
        self.process_name = process_name
        self.date_range = date_range

    def post(self, data, metric):
        url = "{}/{}/{}/_search".format(self.url, self.index, metric)
        result = requests.post(url, data=json.dumps(data))
        return result.json()

    def process(self, data, formatter=None):
        if not formatter:
            def default_formatter(x):
                return x
            formatter = default_formatter

        def processor(result, bucket):
            bucket_max = formatter(bucket["stats"]["max"])
            bucket_min = formatter(bucket["stats"]["min"])
            bucket_avg = formatter(bucket["stats"]["avg"])
            if not result:
                result = {
                    "max": [],
                    "min": [],
                    "avg": [],
                }
            result["max"].append([bucket["key"], bucket_max])
            result["min"].append([bucket["key"], bucket_min])
            result["avg"].append([bucket["key"], bucket_avg])
            return result, bucket_min, bucket_max

        return self.base_process(data, processor)

    def cpu_max(self, interval=None):
        query = self.query(interval=interval)
        response = self.post(query, "cpu_max")
        process = self.process(response)
        return process

    def mem_max(self, interval=None):
        query = self.query(interval=interval)
        return self.process(self.post(query, "mem_max"), formatter=lambda x: x / (1024 * 1024))

    def swap(self, interval=None):
        query = self.query(interval=interval)
        return self.process(self.post(query, "swap"), formatter=lambda x: x / (1024 * 1024))

    def netrx(self, interval=None):
        return self.net_metric("netrx", interval)

    def nettx(self, interval=None):
        return self.net_metric("nettx", interval)

    def net_metric(self, kind, interval=None):
        query = self.query(interval=interval, aggregation=NET_AGGREGATION)
        return self.base_process(self.post(query, kind), self.net_process)

    def net_process(self, result, bucket):
        value = 0
        for in_bucket in bucket["units"]["buckets"]:
            value += in_bucket["delta"]["value"]
        if not result:
            result["requests"] = []
        result["requests"].append([bucket["key"], value])
        return result, value, value

    def units(self, interval=None):
        aggregation = {"units": {"cardinality": {"field": "host"}}}
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "cpu_max"), self.units_process)

    def units_process(self, result, bucket):
        value = bucket["units"]["value"]
        if not result:
            result["units"] = []
        result["units"].append([bucket["key"], value])
        return result, value, value

    def requests_min(self, interval=None):
        aggregation = {"sum": {"sum": {"field": "count"}}}
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "response_time"), self.requests_min_process)

    def requests_min_process(self, result, bucket):
        value = bucket["sum"]["value"]
        if not result:
            result["requests"] = []
        result["requests"].append([bucket["key"], value])
        return result, value, value

    def response_time(self, interval=None):
        aggregation = {
            "stats": {"stats": {"field": "value"}},
            "percentiles": {"percentiles": {"field": "value"}}
        }
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "response_time"), self.response_time_process)

    def response_time_process(self, result, bucket):
        bucket_max = bucket["stats"]["max"]
        bucket_min = bucket["stats"]["min"]
        bucket_avg = bucket["stats"]["avg"]
        if not result:
            result = {
                "max": [],
                "min": [],
                "avg": [],
                "p95": [],
                "p99": [],
            }
        result["max"].append([bucket["key"], bucket_max])
        result["min"].append([bucket["key"], bucket_min])
        result["avg"].append([bucket["key"], bucket_avg])
        result["p95"].append([bucket["key"], bucket["percentiles"]["values"]["95.0"]])
        result["p99"].append([bucket["key"], bucket["percentiles"]["values"]["99.0"]])
        return result, bucket_min, bucket_max

    def top_slow(self, interval=None):
        aggregation = {
            "top": {
                "terms": {
                    "script": "doc['method'].value +'U'+doc['path.raw'].value +'U'+doc['status_code'].value"
                },
                "aggs": {"max": {"max": {"field": "value"}}}
            }
        }
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "response_time"), self.top_slow_process)

    def top_slow_process(self, result, bucket):
        max_response = 0
        for b in bucket["top"]["buckets"]:
            response_time = b["max"]["value"]
            if response_time > max_response:
                max_response = response_time
            r = r'(?P<method>\w+)U(?P<path>.*)U(?P<status_code>\d{3})'
            info = re.search(r, b["key"])
            path = info.groupdict()['path']
            status_code = info.groupdict()['status_code']
            method = info.groupdict()['method']

            if path not in result:
                result[path] = []

            result[path].append([bucket["key"], response_time, status_code, method])

        return result, 0, max_response

    def http_methods(self, interval=None):
        aggregation = {"method": {"terms": {"field": "method"}}}
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "response_time"), self.http_methods_process)

    def http_methods_process(self, result, bucket):
        max_value = 0
        min_value = 0

        for doc in bucket["method"]["buckets"]:
            value = doc["doc_count"]
            method = doc["key"]

            if method not in result:
                result[method] = []

            if value < min_value:
                min_value = value

            if value > max_value:
                max_value = value

            result[method].append([bucket["key"], value])
        return result, min_value, max_value

    def status_code(self, interval=None):
        aggregation = {"status_code": {"terms": {"field": "status_code"}}}
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "response_time"), self.status_code_process)

    def status_code_process(self, result, bucket):
        max_value = 0
        min_value = 0

        for doc in bucket["status_code"]["buckets"]:
            value = doc["doc_count"]
            code = doc["key"]

            if code not in result:
                result[code] = []

            if value < min_value:
                min_value = value

            if value > max_value:
                max_value = value

            result[code].append([bucket["key"], value])
        return result, min_value, max_value

    def connections(self, interval=None):
        aggregation = {
            "connection": {
                "terms": {"script": "doc['connection.raw'].value + doc['connection'].value"}
            }
        }
        query = self.query(interval=interval, aggregation=aggregation)
        return self.base_process(self.post(query, "connection"), self.connections_process)

    def connections_process(self, result, bucket):
        min_value = 0
        max_value = 0
        for doc in bucket["connection"]["buckets"]:
            size = doc["doc_count"]
            conn = doc["key"]
            if conn not in result:
                result[conn] = []

            if size < min_value:
                min_value = size

            if size > max_value:
                max_value = size

            result[conn].append([bucket["key"], size])
        return result, min_value, max_value

    def base_process(self, data, processor):
        result = {}
        min_value = None
        max_value = 0

        if "aggregations" in data and len(data["aggregations"]["date"]["buckets"]) > 0:
            for bucket in data["aggregations"]["date"]["buckets"]:
                result, min_v, max_v = processor(result, bucket)
                if min_value is None or min_v < min_value:
                    min_value = min_v
                if max_v > max_value:
                    max_value = max_v

        return {
            "data": result,
            "min": min_value or 0,
            "max": max_value + 1,
        }

    def query(self, interval=None, aggregation=None):
        if not interval:
            interval = "1m"

        f = {
            "bool": {
                "must": [
                    {
                        "bool": {
                            "should": [
                                {
                                    "term": {
                                        "app": self.app
                                    }
                                },
                                {
                                    "term": {
                                        "app.raw": self.app
                                    }
                                }
                            ]
                        }
                    },
                    {
                        "range": {
                            "@timestamp": {
                                "gte": "now-" + self.date_range,
                                "lt": "now"
                            }
                        },
                    }
                ],
            }
        }

        if self.process_name:
            p = {
                "bool": {
                    "should": [
                        {
                            "term": {
                                "process": self.process_name
                            }
                        },
                        {
                            "term": {
                                "process.raw": self.process_name
                            }
                        }
                    ]
                }
            }
            f["bool"]["must"].append(p)

        query_filter = {
            "filtered": {
                "filter": f
            }
        }
        if not aggregation:
            aggregation = {"stats": {"stats": {"field": "value"}}}
        return {
            "query": query_filter,
            "size": 0,
            "aggs": {
                "date": {
                    "date_histogram": {
                        "field": "@timestamp",
                        "interval": interval
                    },
                    "aggs": aggregation,
                }
            }
        }



from django.conf.urls import url, include
from django.conf import settings

import os


urlpatterns = [
    url(r'^', include('tsuru_dashboard.urls')),
    url(r'^static/(?P<path>.*)$', 'django.views.static.serve',
        {'document_root': os.path.join(settings.BASE_DIR, 'static')})
]



from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import MetricDetail

from mock import patch, Mock


class MetricDetailTest(TestCase):
    @patch("requests.get")
    def setUp(self, requests_mock):
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        self.expected = {
            "name": "app1",
            "framework": "php",
            "repository": "git@git.com:php.git",
            "state": "dead",
            "units": [
                {"Ip": "10.10.10.10"},
                {"Ip": "9.9.9.9"}
            ],
            "teams": ["tsuruteam", "crane"]
        }
        json_mock = Mock(status_code=200)
        json_mock.json.return_value = self.expected
        requests_mock.return_value = json_mock

        self.response = MetricDetail.as_view()(request, app_name="app1")
        self.request = request

    def test_should_use_detail_template(self):
        self.assertIn("apps/metric_details.html", self.response.template_name)

    @patch('requests.get')
    def test_get_envs(self, get):
        expected = [{"name": "DATABASE_HOST", "value": "localhost", "public": True}]
        response_mock = Mock()
        response_mock.json.return_value = expected
        get.return_value = response_mock

        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}

        view = MetricDetail()
        view.request = request
        envs = view.get_envs("appname")

        self.assertListEqual(envs, expected)
        url = '{}/apps/appname/env'.format(settings.TSURU_HOST)
        get.assert_called_with(url, headers={'authorization': 'admin'})

    def test_should_get_the_app_info_from_tsuru(self):
        self.assertDictEqual(self.expected,
                             self.response.context_data["app"])



from django.test import TestCase
from django.forms import CharField

from tsuru_dashboard.teams import forms


class TeamFormTest(TestCase):
    def test_forms_should_have_TeamForm(self):
        self.assertTrue(hasattr(forms, 'TeamForm'))

    def test_team_should_have_name_field(self):
        self.assertIn('name', forms.TeamForm.base_fields)

    def test_name_field_should_instance_of_CharField(self):
        field = forms.TeamForm.base_fields['name']
        self.assertIsInstance(field, CharField)

    def test_name_should_have_at_most_60_characters(self):
        field = forms.TeamForm.base_fields['name']
        self.assertEqual(60, field.max_length)

    def test_validate_name(self):
        invalid_names = [" ", "", "1 my team", "a"]
        valid_names = ["ab", "cd", "team123", "me@myteam.com",
                       "team-1", "team_2", "me+tsuru@me.com"]
        for name in invalid_names:
            form = forms.TeamForm({"name": name})
            self.assertFalse(form.is_valid(),
                             "{} should be invalid".format(name))
        for name in valid_names:
            form = forms.TeamForm({"name": name})
            self.assertTrue(form.is_valid(),
                            "{} should be valid".format(name))



from django.test import TestCase
from django.test import client

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView
from tsuru_dashboard.dashboard import views

import mock
import json
import httpretty


class CloudStatusTest(TestCase):

    def setUp(self):
        self.request = client.RequestFactory().get("/")
        self.request.session = {"tsuru_token": "sometoken"}

    def test_requires_login(self):
        assert issubclass(views.CloudStatusView, LoginRequiredView), \
            "CloudStatusView should inherit from LoginRequiredView"

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_without_access(self, token_is_valid):
        token_is_valid.return_value = True

        url = "{}/docker/node".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, url, status=403)

        url = "{}/apps".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, url, status=403)

        response = views.CloudStatusView.as_view()(self.request)
        data = json.loads(response.content)

        self.assertEqual(0, data["total_nodes"])
        self.assertEqual(0, data["total_apps"])
        self.assertEqual(0, data["total_containers"])
        self.assertEqual(0, data["containers_by_nodes"])

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_total_nodes(self, token_is_valid):
        token_is_valid.return_value = True

        url = "{}/apps".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, url, status=403)

        url = "{}/docker/node".format(settings.TSURU_HOST)
        body = json.dumps({"nodes": [1, 2, 3, 4]})
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        response = views.CloudStatusView.as_view()(self.request)
        data = json.loads(response.content)

        self.assertEqual(4, data["total_nodes"])

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_total_apps_and_containers(self, token_is_valid):
        token_is_valid.return_value = True

        url = "{}/apps".format(settings.TSURU_HOST)
        body = json.dumps([{"units": [1, 2, 3]}])
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        url = "{}/docker/node".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, url, body=body, status=403)

        response = views.CloudStatusView.as_view()(self.request)
        data = json.loads(response.content)

        self.assertEqual(1, data["total_apps"])
        self.assertEqual(3, data["total_containers"])

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_containers_by_node(self, token_is_valid):
        token_is_valid.return_value = True

        url = "{}/apps".format(settings.TSURU_HOST)
        body = json.dumps([{"units": [1, 2, 3, 4, 5, 6]}])
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        url = "{}/docker/node".format(settings.TSURU_HOST)
        body = json.dumps({"nodes": [1, 2]})
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        response = views.CloudStatusView.as_view()(self.request)
        data = json.loads(response.content)

        self.assertEqual(3, data["containers_by_nodes"])



from setuptools import setup, find_packages
from tsuru_dashboard import __version__


setup(
    name="tsuru_dashboard",
    url="https://github.com/tsuru/tsuru-dashboard",
    version=__version__,
    packages=find_packages(),
    description="Web dashboard for tsuru PaaS",
    author="tsuru",
    author_email="tsuru@corp.globo.com",
    include_package_data=True,
    install_requires=[
        "Django>=1.9.3",
        "requests>=2.8.1",
        "python-dateutil>=2.4.2",
        "pytz>=2015.4",
        "Pygments>=2.0.2",
        "tsuruclient>=0.2.1",
        "grequests>=0.2.0",
    ],
)
